'' =================================================================================================
''
''   File....... jm_bam_pp12.spin
''   Purpose.... BAM driver for Propeller Platform "Power 12" Module
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2009-10 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 29 AUG 2009
''   Updated.... 18 SEP 2010
''
'' =================================================================================================

{{

   Reference

   * http://www.artisticlicence.com/WebSiteMaster/App%20Notes/appnote011.pdf
   
}}


con

  LINEAR = 0
  
  
var

  long  cog
    
  byte  brightness[12]                                          ' brightness levels
  

pub init(freq) | okay

'' Start BAM_PP12 driver; uses one cog

  finalize

  ' setup cog parameters before launching

  TIX_001 := (clkfreq / freq) >> 8                              ' 256 steps
  brpntr  := @brightness[0]

  setall(0)                                                     ' clear dimmers

  okay := cog := cognew(@bampp12, 0) + 1                        ' start the bam cog

  return okay 


pub finalize

'' Stops BAM_PP12 driver; frees a cog

  if cog
    cogstop(cog~ - 1)
    setall(0)
    

pub set(chan, level)

'' Sets channel to specified level

  if (chan => 0) & (chan =< 11)
    brightness[chan] := 0 #> level <# 255
    

pub setall(level)

'' Set all channels to same level

  level := 0 #> level <#255                                     ' limit to byte value
  bytefill(@brightness[0], level, 12)


pub ezlog(level)

  level := 0 #> level <#255                                     ' limit to byte value
  
  return (level * level) >> 8                                   ' level := (level^2) / 256


pub setcurve(curvenum, idx)

  idx := 0 #> idx <# 255

  case curvenum
    0     : return curve0[idx]
    1     : return curve0[idx]                                  ' assign new tables here
    2     : return curve0[idx]
    3     : return curve0[idx]
    other : return idx                                          ' default


pub read(ch)

'' Returns channel brightness

  if (ch => 0) & (ch =< 11)
    return brightness[ch]
  else
    return -1


pub address

'' Returns hub address of brightness array

  return @brightness

 
dat

curve0                  byte    000, 001, 002, 003, 004, 005, 006, 007
                        byte    008, 009, 010, 011, 012, 013, 014, 015
                        byte    016, 017, 018, 019, 020, 021, 022, 023
                        byte    024, 025, 026, 027, 028, 029, 030, 031
                        byte    032, 033, 034, 035, 036, 037, 038, 039
                        byte    040, 041, 042, 043, 044, 045, 046, 047
                        byte    048, 049, 050, 051, 052, 053, 054, 055
                        byte    056, 057, 058, 059, 060, 061, 062, 063
                        byte    064, 065, 066, 067, 068, 069, 070, 071
                        byte    072, 073, 074, 075, 076, 077, 078, 079
                        byte    080, 081, 082, 083, 084, 085, 086, 087
                        byte    088, 089, 090, 091, 092, 093, 094, 095
                        byte    096, 097, 098, 099, 100, 101, 102, 103
                        byte    104, 105, 106, 107, 108, 109, 110, 111
                        byte    112, 113, 114, 115, 116, 117, 118, 119
                        byte    120, 121, 122, 123, 124, 125, 126, 127
                        byte    128, 129, 130, 131, 132, 133, 134, 135
                        byte    136, 137, 138, 139, 140, 141, 142, 143
                        byte    144, 145, 146, 147, 148, 149, 150, 151
                        byte    152, 153, 154, 155, 156, 157, 158, 159
                        byte    160, 161, 162, 163, 164, 165, 166, 167
                        byte    168, 169, 170, 171, 172, 173, 174, 175
                        byte    176, 177, 178, 179, 180, 181, 182, 183
                        byte    184, 185, 186, 187, 188, 189, 190, 191
                        byte    192, 193, 194, 195, 196, 197, 198, 199
                        byte    200, 201, 202, 203, 204, 205, 206, 207
                        byte    208, 209, 210, 211, 212, 213, 214, 215
                        byte    216, 217, 218, 219, 220, 221, 222, 223
                        byte    224, 225, 226, 227, 228, 229, 230, 231
                        byte    232, 233, 234, 235, 236, 237, 238, 239
                        byte    240, 241, 242, 243, 244, 245, 246, 247
                        byte    248, 249, 250, 251, 252, 253, 254, 255                         
  
 
dat

                        org     0

bampp12                 mov     outa, #0                        ' clear pins
                        mov     tmp1, #%0011_1111               ' upper group pins
                        shl     tmp1, #16                       ' align upper group to p16
                        or      tmp1, #%0011_1111               ' lower group pins
                        mov     dira, tmp1                      ' set outputs

baminit                 mov     bitperiod, TIX_001
                        min     bitperiod, MIN_TIX              ' keep legal
                        shl     bitperiod, #7                   ' start with MSB
                        mov     bitmask, #%1000_0000
                        
                        mov     bittimer, bitperiod             ' initialize bit timer
                        add     bittimer, cnt                   ' sync with system counter

bamloop                 mov     tmpouts, #0                     ' clear outputs workspace
                        mov     chmask, MASK_LO                 ' start with low group
                        
bamout                  test    level00, bitmask        wc      ' test bit
                        muxc    tmpouts, chmask                 ' write bit to workspace
                        shl     chmask, #1                      ' next channel
                        test    level01, bitmask        wc   
                        muxc    tmpouts, chmask              
                        shl     chmask, #1
                        test    level02, bitmask        wc   
                        muxc    tmpouts, chmask              
                        shl     chmask, #1
                        test    level03, bitmask        wc   
                        muxc    tmpouts, chmask              
                        shl     chmask, #1
                        test    level04, bitmask        wc   
                        muxc    tmpouts, chmask              
                        shl     chmask, #1
                        test    level05, bitmask        wc   
                        muxc    tmpouts, chmask
                        mov     chmask, MASK_HI                 ' switch to high group
                        test    level06, bitmask        wc   
                        muxc    tmpouts, chmask              
                        shl     chmask, #1
                        test    level07, bitmask        wc   
                        muxc    tmpouts, chmask              
                        shl     chmask, #1
                        test    level08, bitmask        wc   
                        muxc    tmpouts, chmask              
                        shl     chmask, #1
                        test    level09, bitmask        wc   
                        muxc    tmpouts, chmask              
                        shl     chmask, #1
                        test    level10, bitmask        wc   
                        muxc    tmpouts, chmask              
                        shl     chmask, #1
                        test    level11, bitmask        wc   
                        muxc    tmpouts, chmask              
                        mov     outa, tmpouts                   ' update outputs

getlevels               test    bitmask, #%1000_0000    wc      ' on bit7?
        if_nc           jmp     #xferlevels                     ' if no, skip hub access                                      

                        mov     tmp1, brpntr                    ' point to brightness[0]
                        rdbyte  tmpbr00, tmp1                   ' read brightness
                        add     tmp1, #1                        ' point to next element
                        rdbyte  tmpbr01, tmp1 
                        add     tmp1, #1
                        rdbyte  tmpbr02, tmp1 
                        add     tmp1, #1          
                        rdbyte  tmpbr03, tmp1 
                        add     tmp1, #1
                        rdbyte  tmpbr04, tmp1      
                        add     tmp1, #1
                        rdbyte  tmpbr05, tmp1
                        add     tmp1, #1
                        rdbyte  tmpbr06, tmp1
                        add     tmp1, #1
                        rdbyte  tmpbr07, tmp1
                        add     tmp1, #1
                        rdbyte  tmpbr08, tmp1
                        add     tmp1, #1
                        rdbyte  tmpbr09, tmp1
                        add     tmp1, #1
                        rdbyte  tmpbr10, tmp1
                        add     tmp1, #1
                        rdbyte  tmpbr11, tmp1                      

xferlevels              test    bitmask, #%0000_0001    wc      ' on bit0?
        if_nc           jmp     #nextbit                        ' if no, skip transfer

                        mov     level00, tmpbr00                ' transfer to working regs
                        mov     level01, tmpbr01
                        mov     level02, tmpbr02
                        mov     level03, tmpbr03
                        mov     level04, tmpbr04
                        mov     level05, tmpbr05
                        mov     level06, tmpbr06
                        mov     level07, tmpbr07
                        mov     level08, tmpbr08
                        mov     level09, tmpbr09
                        mov     level10, tmpbr10
                        mov     level11, tmpbr11

nextbit                 shr     bitmask, #1             wz      ' next bit
        if_nz           shr     bitperiod, #1                   ' update bit period
        if_nz           jmp     #looppad
                                         
newcycle                mov     bitperiod, TIX_001
                        min     bitperiod, MIN_TIX              
                        shl     bitperiod, #7
                        mov     bitmask, #%1000_0000

looppad                 waitcnt bittimer, bitperiod             ' finish bit, then reload
                        jmp     #bamloop
                        
' --------------------------------------------------------------------------------------------------

MIN_TIX                 long    400                             ' minimum tix per bit                         ' 
BIT0                    long    |< 0                            ' set bit0 to 1

MASK_LO                 long    |< 0                            ' base pin masks
MASK_HI                 long    |< 16

TIX_001                 long    0-0                             ' timing for bit0  

brpntr                  long    0-0                             ' hub address of brighness array
ltpntr                  long    0-0

level00                 long    0                               ' brightness levels
level01                 long    0
level02                 long    0
level03                 long    0
level04                 long    0
level05                 long    0
level06                 long    0                               ' brightness levels
level07                 long    0
level08                 long    0
level09                 long    0
level10                 long    0
level11                 long    0

tmpbr00                 res     1                               ' temporary holding regs
tmpbr01                 res     1
tmpbr02                 res     1
tmpbr03                 res     1
tmpbr04                 res     1
tmpbr05                 res     1
tmpbr06                 res     1                               ' temporary holding regs
tmpbr07                 res     1
tmpbr08                 res     1
tmpbr09                 res     1
tmpbr10                 res     1
tmpbr11                 res     1

tmpouts                 res     1                               ' outputs workspace
chmask                  res     1                               ' channel mask

bitmask                 res     1                               ' bit under test
bitperiod               res     1                               ' duration of current bit
bittimer                res     1

tmp1                    res     1
tmp2                    res     1

                        fit     492
                        

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}

                