'' =================================================================================================
''
''   File....... jm_pp12_kinara.spin
''   Purpose.... Electronic Kwanzaa Kinara using the P/P Power 12 module
''   Author..... Jon "JonnyMac" McPhalen 
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 19 SEP 2010
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
' _xinfreq = 6_250_000   

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  ' for Propeller Platform PP12 module

  RX1   = 31                                                    ' programming port
  TX1   = 30
  
  SDA   = 29                                                    ' i2c port
  SCL   = 28

  RX2   = 27                                                    ' ttl serial
  TX2   = 26

  OUT12 = 21                                                    ' high group
  OUT11 = 20
  OUT10 = 19
  OUT09 = 18
  OUT08 = 17
  OUT07 = 16

  TRIG  = 06                                                    ' trigger input

  OUT06 = 05                                                    ' low group
  OUT05 = 04
  OUT04 = 03
  OUT03 = 02
  OUT02 = 01
  OUT01 = 00  


con

  #0, ACTIVE_LOW, ACTIVE_HIGH                                   ' input modes

  TR_MODE = ACTIVE_HIGH                                         ' trigger pin mode

  
obj

  leds : "jm_bam_pp12"


var

  byte  wicks[12]                                               ' candle levels
  
  long  stack[64]                                               ' for flicker cog
  

dat

kinara          byte    3, 0, 6, 1, 5, 2, 4                     ' remap to Kinara



pub main | day, level, idx 

  leds.init(360)                                                ' start LED

  bytefill(@wicks, 0, 12)                                       ' all off to start
  cognew(flicker(7), @stack)                                    ' start flicker generator  

  pause(5)                                                      ' let everything start

  day := 0
  
  repeat
    repeat
    until (getbutton(TRIG, 100) == true)                        ' wait for press
    pause(250)                                                  
    if (getbutton(TRIG, 150) == false)                          ' if not still pressed
      if (day < 7)                                              ' and not all lit
        idx := kinara[day]                                      ' remap to candle output
        repeat level from 0 to 255                              ' light candle
          wicks[idx] := level  
          pause(6)
        ++day                                                   ' update day
    else
      repeat level from 255 to 0
        repeat day from 6 to 0                                  ' loop through all
          if (wicks[day] > 0)                                   ' if lit
            wicks[day] := level                                 '   dim down
        pause(24)                                               ' slow fade-out
      quit 

  repeat ' until shutdown
    

pub getcount(pin, cmax) | count, idle

'' Counts valid presses on 'pin' up to 'cmax'

  count := 0
  idle := 0
  
  repeat
    if (getbutton(pin, 50) == true)                             ' wait for input
      repeat
      until (getbutton(pin, 50) == false)                       ' force release 
      count += 1                                                ' update press count
      idle := 0                                                 ' reset idle timer
    else
      idle += 50                                                ' update idle if no press
        
    if (count == cmax)                                          ' abort on max day
      quit

    if (count > 0) and (idle => 1000)                           ' abort when done
      quit
  
  return count


pub getbutton(pin, dbtime) | db, t

'' Debounces pushbutton input on 'pin' for 'dbtime' milliseconds

  dira[pin] := 0                                                ' force to input

  db := 0                                                       ' clear debounce counter

  t := cnt                                                      ' sync with system cnt  
  repeat dbtime
    waitcnt(t += MS_001)                                        ' wait 1ms
    if (TR_MODE == ACTIVE_HIGH)                                 ' udpate db count  
      db += ina[pin]
    else
      db += !ina[pin] & %1                                        

  return (db == dbtime)

     
pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)


pub getbit(value, pos)

'  Returns value.pos (0 or 1)

  return (value >> pos) & 1 


pub setbit(value, pos, bit)

' returns value with bit.0 in value.pos

  if ((bit & 1) == 1)
    return value | (1 << pos)
  else
    return value & !(1 << pos)


pub togglebit(value, pos)

' returns value with value.pos flipped

  return value ^ (1 << pos)
    

pri flicker(wix) | lotto, idx, tmp, {
            } target[12], base[12], direction

'' Candle flicker generator
'' -- launch into separate cog

  lotto := cnt                                                  ' seed random #
  repeat idx from 0 to 11
    target[idx] := ?lotto & $FF | $80                           ' new target, 128 to 255 
    base[idx] := 0
  direction := %0000_0000_0000                                  ' ramp up to start
          
  repeat
    ?lotto                                           
    repeat idx from 0 to (wix-1)                                ' loop through channels
      if (base[idx] < target[idx])                              ' base less than target?                    
        ++base[idx]                                             '   yes, inc base
      elseif (base[idx] > target[idx])                          ' base greater than target?
        --base[idx]                                             '   yes, dec base
      else                                                      ' at target
        direction := togglebit(direction, idx)                  ' flip ramp direction
        if (getbit(direction, idx) == 0)                        ' going up?
          target[idx] := ?lotto & $FF | $80 
        else
          target[idx] := ?lotto & $7F 
        
      tmp := (base[idx] * wicks[idx]) >> 8                      ' calculate final base brightess              
      leds.set(idx, tmp)                                        ' set base LED

    waitcnt(constant(20 * US_001) + cnt)
       

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}