'' =================================================================================================
''
''   File....... jm_pp12_renard_ez.spin
''   Purpose.... Renard protocol receiver for P/P Power 12 module -- dimmer mode
''   Author..... Jon "JonnyMac" McPhalen 
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 19 SEP 2010
''
'' =================================================================================================

{
   Code based on protocol description found here:
   -- http://computerchristmas.com/?link=how_to&HowToId=71

   This program does not rebroadcast values; if used on tranditional, multi-drop
   RS-485 system this is not necessary.
}


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
' _xinfreq = 6_250_000   

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  ' for Propeller Platform PP12 module

  RX1   = 31                                                    ' programming port
  TX1   = 30
  
  SDA   = 29                                                    ' i2c port
  SCL   = 28

  RX2   = 27                                                    ' ttl serial
  TX2   = 26

  OUT12 = 21                                                    ' high group
  OUT11 = 20
  OUT10 = 19
  OUT09 = 18
  OUT08 = 17
  OUT07 = 16

  TRIG  = 06                                                    ' trigger input

  OUT06 = 05                                                    ' low group
  OUT05 = 04
  OUT04 = 03
  OUT03 = 02
  OUT02 = 01
  OUT01 = 00  


con

  ADDR = $80                                                    ' board address

  PAD  = $7D
  SYNC = $7E
  ESC  = $7F

   #0, W_SYNC, W_ADDR, W_LVL1, W_LVL2               ' rx states
  #12, CHANNELS

  
obj

  leds    : "jm_bam_pp12"
  console : "fullduplexserial"


var


pub main | state, cmd, chan   

  leds.init(360)                                                ' start LED
  console.start(RX1, TX1, %0000, 115_200)                       ' start serial input

  pause(5)                                                      ' let everything start
  console.rxflush                                               ' clear RX buffer
  state := W_SYNC
  chan := 0

  repeat
    cmd := console.rxtime(1000)                                 ' wait for byte
    if (cmd == -1)                                              ' if timeout
      leds.setall(0)                                            ' clear outputs
      state := W_SYNC                                           ' reset state
      
    case state
      W_SYNC : if (cmd == SYNC)
                 state := W_ADDR
            
      W_ADDR : if (cmd == PAD)
                 ' ignore
               elseif (cmd == SYNC)
                 ' no change
               elseif (cmd == ADDR)
                 state := W_LVL1
                 chan := 0
               else
                 state := W_SYNC                  
                                                   
      W_LVL1 : if (cmd == PAD)
                 ' ignore it
               elseif (cmd == SYNC)
                 state := W_ADDR
               elseif (cmd == ESC)
                 state := W_LVL2
               else
                 leds.set(chan, cmd) 
                 if (++chan == CHANNELS)              
                   state := W_SYNC                
            
      W_LVL2 : if (cmd => $2F) and (cmd =< $31)
                 cmd := $7D + (cmd-$2F)
                 leds.set(chan, cmd) 
                 if (++chan == CHANNELS)              
                   state := W_SYNC     
               elseif (cmd == SYNC)
                 state := W_ADDR
               else
                 state := W_SYNC
      
      
      
pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)

                       
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}